@@ -8,6 +8,10 @@ describe "Range#last" do
     (1..5).last(3).should == [3, 4, 5]
   end
 
+  it "returns the specified number if elements for single element inclusive range" do
+    (1..1).last(1).should == [1]
+  end
+
   it "returns an empty array for an empty Range" do
     (0...0).last(2).should == []
   end
@@ -21,7 +25,7 @@ describe "Range#last" do
   end
 
   it "raises an ArgumentError when count is negative" do
-    lambda { (0..2).last(-1) }.should raise_error(ArgumentError)
+    -> { (0..2).last(-1) }.should raise_error(ArgumentError)
   end
 
   it "calls #to_int to convert the argument" do
@@ -32,7 +36,7 @@ describe "Range#last" do
   it "raises a TypeError if #to_int does not return an Integer" do
     obj = mock("to_int")
     obj.should_receive(:to_int).and_return("1")
-    lambda { (2..3).last(obj) }.should raise_error(TypeError)
+    -> { (2..3).last(obj) }.should raise_error(TypeError)
   end
 
   it "truncates the value when passed a Float" do
@@ -40,10 +44,14 @@ describe "Range#last" do
   end
 
   it "raises a TypeError when passed nil" do
-    lambda { (2..3).last(nil) }.should raise_error(TypeError)
+    -> { (2..3).last(nil) }.should raise_error(TypeError)
   end
 
   it "raises a TypeError when passed a String" do
-    lambda { (2..3).last("1") }.should raise_error(TypeError)
+    -> { (2..3).last("1") }.should raise_error(TypeError)
+  end
+
+  it "raises a RangeError when called on an endless range" do
+    -> { eval("(1..)").last }.should raise_error(RangeError)
   end
 end
