@@ -97,7 +97,8 @@ describe "An instance method" do
         def foo; end
       end
     }.should raise_error(FrozenError) { |e|
-      e.message.should.start_with? "can't modify frozen module"
+      msg_class = ruby_version_is("4.0") ? "Module" : "module"
+      e.message.should == "can't modify frozen #{msg_class}: #{e.receiver}"
     }
 
     -> {
@@ -106,7 +107,8 @@ describe "An instance method" do
         def foo; end
       end
     }.should raise_error(FrozenError){ |e|
-      e.message.should.start_with? "can't modify frozen class"
+      msg_class = ruby_version_is("4.0") ? "Class" : "class"
+      e.message.should == "can't modify frozen #{msg_class}: #{e.receiver}"
     }
   end
 end
@@ -197,26 +199,8 @@ describe "An instance method with a default argument" do
     foo(2,3,3).should == [2,3,[3]]
   end
 
-  ruby_version_is ''...'2.7' do
-    it "warns and uses a nil value when there is an existing local method with same name" do
-      def bar
-        1
-      end
-      -> {
-        eval "def foo(bar = bar)
-          bar
-        end"
-      }.should complain(/circular argument reference/)
-      foo.should == nil
-      foo(2).should == 2
-    end
-  end
-
-  ruby_version_is '2.7' do
-    it "raises a syntaxError an existing method with the same name as the local variable" do
-      def bar
-        1
-      end
+  ruby_version_is ""..."3.4" do
+    it "raises a SyntaxError if using the argument in its default value" do
       -> {
         eval "def foo(bar = bar)
           bar
@@ -225,6 +209,17 @@ describe "An instance method with a default argument" do
     end
   end
 
+  ruby_version_is "3.4" do
+    it "is nil if using the argument in its default value" do
+      -> {
+        eval "def foo(bar = bar)
+          bar
+        end
+        foo"
+      }.call.should == nil
+    end
+  end
+
   it "calls a method with the same name as the local when explicitly using ()" do
     def bar
       1
@@ -255,7 +250,7 @@ describe "A singleton method definition" do
   end
 
   it "can be declared for a global variable" do
-    $__a__ = "hi"
+    $__a__ = +"hi"
     def $__a__.foo
       7
     end
@@ -290,20 +285,21 @@ describe "A singleton method definition" do
   it "raises FrozenError with the correct class name" do
     obj = Object.new
     obj.freeze
-    -> { def obj.foo; end }.should raise_error(FrozenError){ |e|
-      e.message.should.start_with? "can't modify frozen object"
-    }
+    msg_class = ruby_version_is("4.0") ? "Object" : "object"
+    -> { def obj.foo; end }.should raise_error(FrozenError, "can't modify frozen #{msg_class}: #{obj}")
 
+    obj = Object.new
     c = obj.singleton_class
-    -> { def c.foo; end }.should raise_error(FrozenError){ |e|
-      e.message.should.start_with? "can't modify frozen Class"
-    }
+    c.singleton_class.freeze
+    -> { def c.foo; end }.should raise_error(FrozenError, "can't modify frozen Class: #{c}")
+
+    c = Class.new
+    c.freeze
+    -> { def c.foo; end }.should raise_error(FrozenError, "can't modify frozen Class: #{c}")
 
     m = Module.new
     m.freeze
-    -> { def m.foo; end }.should raise_error(FrozenError){ |e|
-      e.message.should.start_with? "can't modify frozen Module"
-    }
+    -> { def m.foo; end }.should raise_error(FrozenError, "can't modify frozen Module: #{m}")
   end
 end
 
@@ -531,6 +527,8 @@ describe "A nested method definition" do
 
     obj = DefSpecNested.new
     obj.inherited_method.should == obj
+  ensure
+    DefSpecNested.send(:remove_const, :TARGET)
   end
 
   # See http://yugui.jp/articles/846#label-3
@@ -552,6 +550,8 @@ describe "A nested method definition" do
 
     DefSpecNested.should_not have_instance_method :arg_method
     DefSpecNested.should_not have_instance_method :body_method
+  ensure
+    DefSpecNested.send(:remove_const, :OBJ)
   end
 
   it "creates an instance method inside Class.new" do
