@@ -14,69 +14,34 @@ describe "Evaluation order during assignment" do
   end
 
   context "with multiple assignment" do
-    ruby_version_is ""..."3.1" do
-      it "does not evaluate from left to right" do
-        obj = VariablesSpecs::EvalOrder.new
-
-        obj.instance_eval do
-          foo[0], bar.baz = a, b
-        end
-
-        obj.order.should == ["a", "b", "foo", "foo[]=", "bar", "bar.baz="]
+    it "evaluates from left to right, receivers first then methods" do
+      obj = VariablesSpecs::EvalOrder.new
+      obj.instance_eval do
+        foo[0], bar.baz = a, b
       end
 
-      it "cannot be used to swap variables with nested method calls" do
-        node = VariablesSpecs::EvalOrder.new.node
-
-        original_node = node
-        original_node_left = node.left
-        original_node_left_right = node.left.right
-
-        node.left, node.left.right, node = node.left.right, node, node.left
-        # Should evaluate in the order of:
-        # RHS: node.left.right, node, node.left
-        # LHS:
-        # * node(original_node), original_node.left = original_node_left_right
-        # * node(original_node), node.left(changed in the previous assignment to original_node_left_right),
-        #   original_node_left_right.right = original_node
-        # * node = original_node_left
-
-        node.should == original_node_left
-        node.right.should_not == original_node
-        node.right.left.should_not == original_node_left_right
-      end
+      obj.order.should == ["foo", "bar", "a", "b", "foo[]=", "bar.baz="]
     end
 
-    ruby_version_is "3.1" do
-      it "evaluates from left to right, receivers first then methods" do
-        obj = VariablesSpecs::EvalOrder.new
-        obj.instance_eval do
-          foo[0], bar.baz = a, b
-        end
+    it "can be used to swap variables with nested method calls" do
+      node = VariablesSpecs::EvalOrder.new.node
 
-        obj.order.should == ["foo", "bar", "a", "b", "foo[]=", "bar.baz="]
-      end
+      original_node = node
+      original_node_left = node.left
+      original_node_left_right = node.left.right
 
-      it "can be used to swap variables with nested method calls" do
-        node = VariablesSpecs::EvalOrder.new.node
+      node.left, node.left.right, node = node.left.right, node, node.left
+      # Should evaluate in the order of:
+      # LHS: node, node.left(original_node_left)
+      # RHS: original_node_left_right, original_node, original_node_left
+      # Ops:
+      # * node(original_node), original_node.left = original_node_left_right
+      # * original_node_left.right = original_node
+      # * node = original_node_left
 
-        original_node = node
-        original_node_left = node.left
-        original_node_left_right = node.left.right
-
-        node.left, node.left.right, node = node.left.right, node, node.left
-        # Should evaluate in the order of:
-        # LHS: node, node.left(original_node_left)
-        # RHS: original_node_left_right, original_node, original_node_left
-        # Ops:
-        # * node(original_node), original_node.left = original_node_left_right
-        # * original_node_left.right = original_node
-        # * node = original_node_left
-
-        node.should == original_node_left
-        node.right.should == original_node
-        node.right.left.should == original_node_left_right
-      end
+      node.should == original_node_left
+      node.right.should == original_node
+      node.right.left.should == original_node_left_right
     end
   end
 end
@@ -381,6 +346,9 @@ describe "Multiple assignment" do
         SINGLE_RHS_1, SINGLE_RHS_2 = 1
         [SINGLE_RHS_1, SINGLE_RHS_2].should == [1, nil]
       end
+    ensure
+      VariableSpecs.send(:remove_const, :SINGLE_RHS_1)
+      VariableSpecs.send(:remove_const, :SINGLE_RHS_2)
     end
   end
 
@@ -395,11 +363,22 @@ describe "Multiple assignment" do
       a.should == []
     end
 
-    it "calls #to_a to convert nil to an empty Array" do
-      nil.should_receive(:to_a).and_return([])
+    ruby_version_is "4.0" do
+      it "converts nil to empty array without calling a method" do
+        nil.should_not_receive(:to_a)
 
-      (*a = *nil).should == []
-      a.should == []
+        (*a = *nil).should == []
+        a.should == []
+      end
+    end
+
+    ruby_version_is ""..."4.0"  do
+      it "calls #to_a to convert nil to an empty Array" do
+        nil.should_receive(:to_a).and_return([])
+
+        (*a = *nil).should == []
+        a.should == []
+      end
     end
 
     it "does not call #to_a on an Array" do
@@ -619,6 +598,8 @@ describe "Multiple assignment" do
         (*SINGLE_SPLATTED_RHS) = *1
         SINGLE_SPLATTED_RHS.should == [1]
       end
+    ensure
+      VariableSpecs.send(:remove_const, :SINGLE_SPLATTED_RHS)
     end
   end
 
@@ -818,6 +799,9 @@ describe "Multiple assignment" do
         MRHS_VALUES_1.should == 1
         MRHS_VALUES_2.should == 2
       end
+    ensure
+      VariableSpecs.send(:remove_const, :MRHS_VALUES_1)
+      VariableSpecs.send(:remove_const, :MRHS_VALUES_2)
     end
 
     it "assigns all RHS values as an array to a single LHS constant" do
@@ -825,6 +809,8 @@ describe "Multiple assignment" do
         MRHS_VALUES = 1, 2, 3
         MRHS_VALUES.should == [1, 2, 3]
       end
+    ensure
+      VariableSpecs.send(:remove_const, :MRHS_VALUES)
     end
   end
 
