@@ -1,12 +1,10 @@
 require_relative "../../../spec_helper"
 platform_is :windows do
-  verbose, $VERBOSE = $VERBOSE, nil
-
   require 'win32ole'
 
-  describe "WIN32OLE_TYPE#helpstring for Shell Controls" do
+  describe "WIN32OLE::Type#helpstring for Shell Controls" do
     before :each do
-      @ole_type = WIN32OLE_TYPE.new("Microsoft Shell Controls And Automation", "Shell")
+      @ole_type = WIN32OLE::Type.new("Microsoft Shell Controls And Automation", "Shell")
     end
 
     after :each do
@@ -18,6 +16,4 @@ platform_is :windows do
     end
 
   end
-ensure
-  $VERBOSE = verbose
 end
