@@ -1,14 +1,12 @@
 require_relative "../../../spec_helper"
 platform_is :windows do
-  verbose, $VERBOSE = $VERBOSE, nil
-
   require 'win32ole'
 
   describe "WIN32OLE_VARIABLE#visible?" do
     # not sure how WIN32OLE_VARIABLE objects are supposed to be generated
     # WIN32OLE_VARIABLE.new even seg faults in some cases
     before :each do
-      ole_type = WIN32OLE_TYPE.new("Microsoft Shell Controls And Automation", "ShellSpecialFolderConstants")
+      ole_type = WIN32OLE::Type.new("Microsoft Shell Controls And Automation", "ShellSpecialFolderConstants")
       @var = ole_type.variables[0]
     end
 
@@ -18,6 +16,4 @@ platform_is :windows do
 
   end
 
-ensure
-  $VERBOSE = verbose
 end
