@@ -108,16 +108,13 @@ class OpenSSL::TestCipher < OpenSSL::TestCase
     assert_not_equal s1, s2
   end
 
-  def test_empty_data
-    cipher = OpenSSL::Cipher.new("DES-EDE3-CBC").encrypt
-    cipher.random_key
-    assert_raise(ArgumentError) { cipher.update("") }
-  end
-
   def test_initialize
     cipher = OpenSSL::Cipher.new("DES-EDE3-CBC")
     assert_raise(RuntimeError) { cipher.__send__(:initialize, "DES-EDE3-CBC") }
     assert_raise(RuntimeError) { OpenSSL::Cipher.allocate.final }
+    assert_raise(OpenSSL::Cipher::CipherError) {
+      OpenSSL::Cipher.new("no such algorithm")
+    }
   end
 
   def test_ctr_if_exists
@@ -134,26 +131,47 @@ class OpenSSL::TestCipher < OpenSSL::TestCase
     assert_equal pt, cipher.update(ct) << cipher.final
   end
 
+  def test_update_with_buffer
+    cipher = OpenSSL::Cipher.new("aes-128-ecb").encrypt
+    cipher.random_key
+    expected = cipher.update("data") << cipher.final
+    assert_equal 16, expected.bytesize
+
+    # Buffer is supplied
+    cipher.reset
+    buf = String.new
+    assert_same buf, cipher.update("data", buf)
+    assert_equal expected, buf + cipher.final
+
+    # Buffer is frozen
+    cipher.reset
+    assert_raise(FrozenError) { cipher.update("data", String.new.freeze) }
+
+    # Buffer is a shared string [ruby-core:120141] [Bug #20937]
+    cipher.reset
+    buf = "x" * 1024
+    shared = buf[-("data".bytesize + 32)..-1]
+    assert_same shared, cipher.update("data", shared)
+    assert_equal expected, shared + cipher.final
+  end
+
   def test_ciphers
-    OpenSSL::Cipher.ciphers.each{|name|
-      next if /netbsd/ =~ RUBY_PLATFORM && /idea|rc5/i =~ name
-      begin
-        assert_kind_of(OpenSSL::Cipher, OpenSSL::Cipher.new(name))
-      rescue OpenSSL::Cipher::CipherError => e
-        raise unless /wrap/ =~ name and /wrap mode not allowed/ =~ e.message
-      end
-    }
+    ciphers = OpenSSL::Cipher.ciphers
+    assert_kind_of Array, ciphers
+    assert_include ciphers, "aes-128-cbc"
+    assert_include ciphers, "aes128" # alias of aes-128-cbc
+    assert_include ciphers, "aes-128-gcm"
   end
 
   def test_AES
     pt = File.read(__FILE__)
-    %w(ECB CBC CFB OFB).each{|mode|
-      c1 = OpenSSL::Cipher.new("AES-256-#{mode}")
+    %w(ecb cbc cfb ofb).each{|mode|
+      c1 = OpenSSL::Cipher.new("aes-256-#{mode}")
       c1.encrypt
       c1.pkcs5_keyivgen("passwd")
       ct = c1.update(pt) + c1.final
 
-      c2 = OpenSSL::Cipher.new("AES-256-#{mode}")
+      c2 = OpenSSL::Cipher.new("aes-256-#{mode}")
       c2.decrypt
       c2.pkcs5_keyivgen("passwd")
       assert_equal(pt, c2.update(ct) + c2.final)
@@ -163,10 +181,14 @@ class OpenSSL::TestCipher < OpenSSL::TestCase
   def test_update_raise_if_key_not_set
     assert_raise(OpenSSL::Cipher::CipherError) do
       # it caused OpenSSL SEGV by uninitialized key [Bug #2768]
-      OpenSSL::Cipher.new("AES-128-ECB").update "." * 17
+      OpenSSL::Cipher.new("aes-128-ecb").update "." * 17
     end
   end
 
+  def test_auth_tag_error_inheritance
+    assert_equal OpenSSL::Cipher::CipherError, OpenSSL::Cipher::AuthTagError.superclass
+  end
+
   def test_authenticated
     cipher = OpenSSL::Cipher.new('aes-128-gcm')
     assert_predicate(cipher, :authenticated?)
@@ -197,7 +219,8 @@ class OpenSSL::TestCipher < OpenSSL::TestCase
     cipher = new_decryptor("aes-128-ccm", **kwargs, ccm_data_len: ct.length, auth_tag: tag[0, 8], auth_data: aad)
     assert_equal pt, cipher.update(ct) << cipher.final
 
-    # wrong tag is rejected
+    # wrong tag is rejected - in CCM, authentication happens during update, but
+    # we consider this a general CipherError since update failures can have various causes
     tag2 = tag.dup
     tag2.setbyte(-1, (tag2.getbyte(-1) + 1) & 0xff)
     cipher = new_decryptor("aes-128-ccm", **kwargs, ccm_data_len: ct.length, auth_tag: tag2, auth_data: aad)
@@ -212,15 +235,9 @@ class OpenSSL::TestCipher < OpenSSL::TestCase
     ct2 = ct[0..-2] << ct[-1].succ
     cipher = new_decryptor("aes-128-ccm", **kwargs, ccm_data_len: ct2.length, auth_tag: tag, auth_data: aad)
     assert_raise(OpenSSL::Cipher::CipherError) { cipher.update(ct2) }
-  rescue OpenSSL::Cipher::CipherError
-    if /mingw/i =~ RUBY_PLATFORM
-      omit "skip on OpenSSL::Cipher::CipherError from 'ccm_data_len=': Maybe it's because of OpenSSL in MinGW"
-    else
-      raise
-    end
   end if has_cipher?("aes-128-ccm") &&
          OpenSSL::Cipher.new("aes-128-ccm").authenticated? &&
-         OpenSSL::OPENSSL_VERSION_NUMBER >= 0x1010103f # version >= 1.1.1c
+         openssl?(1, 1, 1, 0x03, 0xf) # version >= 1.1.1c
 
   def test_aes_gcm
     # GCM spec Appendix B Test Case 4
@@ -256,19 +273,19 @@ class OpenSSL::TestCipher < OpenSSL::TestCase
     tag2.setbyte(-1, (tag2.getbyte(-1) + 1) & 0xff)
     cipher = new_decryptor("aes-128-gcm", key: key, iv: iv, auth_tag: tag2, auth_data: aad)
     cipher.update(ct)
-    assert_raise(OpenSSL::Cipher::CipherError) { cipher.final }
+    assert_raise(OpenSSL::Cipher::AuthTagError) { cipher.final }
 
     # wrong aad is rejected
     aad2 = aad[0..-2] << aad[-1].succ
     cipher = new_decryptor("aes-128-gcm", key: key, iv: iv, auth_tag: tag, auth_data: aad2)
     cipher.update(ct)
-    assert_raise(OpenSSL::Cipher::CipherError) { cipher.final }
+    assert_raise(OpenSSL::Cipher::AuthTagError) { cipher.final }
 
     # wrong ciphertext is rejected
     ct2 = ct[0..-2] << ct[-1].succ
     cipher = new_decryptor("aes-128-gcm", key: key, iv: iv, auth_tag: tag, auth_data: aad)
     cipher.update(ct2)
-    assert_raise(OpenSSL::Cipher::CipherError) { cipher.final }
+    assert_raise(OpenSSL::Cipher::AuthTagError) { cipher.final }
   end
 
   def test_aes_gcm_variable_iv_len
@@ -328,6 +345,24 @@ class OpenSSL::TestCipher < OpenSSL::TestCase
 
   end if has_cipher?("aes-128-ocb")
 
+  def test_aes_gcm_siv
+    # RFC 8452 Appendix C.1., 8th example
+    key = ["01000000000000000000000000000000"].pack("H*")
+    iv  = ["030000000000000000000000"].pack("H*")
+    aad = ["01"].pack("H*")
+    pt =  ["0200000000000000"].pack("H*")
+    ct =  ["1e6daba35669f4273b0a1a2560969cdf790d99759abd1508"].pack("H*")
+    tag = ["3b0a1a2560969cdf790d99759abd1508"].pack("H*")
+    ct_without_tag = ct.byteslice(0, ct.bytesize - tag.bytesize)
+
+    cipher = new_encryptor("aes-128-gcm-siv", key: key, iv: iv, auth_data: aad)
+    assert_equal ct_without_tag, cipher.update(pt) << cipher.final
+    assert_equal tag, cipher.auth_tag
+    cipher = new_decryptor("aes-128-gcm-siv", key: key, iv: iv, auth_tag: tag,
+                           auth_data: aad)
+    assert_equal pt, cipher.update(ct_without_tag) << cipher.final
+  end if openssl?(3, 2, 0)
+
   def test_aes_gcm_key_iv_order_issue
     pt = "[ruby/openssl#49]"
     cipher = OpenSSL::Cipher.new("aes-128-gcm").encrypt
@@ -346,6 +381,22 @@ class OpenSSL::TestCipher < OpenSSL::TestCase
     assert_equal tag1, tag2
   end
 
+  def test_aes_keywrap_pad
+    # RFC 5649 Section 6; The second example
+    kek = ["5840df6e29b02af1ab493b705bf16ea1ae8338f4dcc176a8"].pack("H*")
+    key = ["466f7250617369"].pack("H*")
+    wrap = ["afbeb0f07dfbf5419200f2ccb50bb24f"].pack("H*")
+
+    begin
+      cipher = OpenSSL::Cipher.new("id-aes192-wrap-pad").encrypt
+    rescue OpenSSL::Cipher::CipherError
+      omit "id-aes192-wrap-pad is not supported: #$!"
+    end
+    cipher.key = kek
+    ct = cipher.update(key) << cipher.final
+    assert_equal wrap, ct
+  end
+
   def test_non_aead_cipher_set_auth_data
     assert_raise(OpenSSL::Cipher::CipherError) {
       cipher = OpenSSL::Cipher.new("aes-128-cfb").encrypt
