@@ -480,6 +480,13 @@ class TestRipper::ParserEvents < Test::Unit::TestCase
     assert_equal true, thru_call
     assert_equal "[call(vcall(foo),.,call,[])]", tree
 
+    thru_call = false
+    assert_nothing_raised {
+      tree = parse("a b do end.()", :on_call) {thru_call = true}
+    }
+    assert_equal true, thru_call
+    assert_equal "[call(command(a,[vcall(b)],&do_block(,bodystmt([void()]))),.,call,[])]", tree
+
     thru_call = false
     assert_nothing_raised {
       tree = parse("self::foo", :on_call) {thru_call = true}
